@@ -1,11 +1,11 @@
 """Core routines."""
 
-from .common import Common, DataObject
+from .dataset import DataSet, DataObject
 from .composite import MultiBlock
 from .datasetattributes import DataSetAttributes
 from .filters import (CompositeFilters, DataSetFilters, PolyDataFilters,
                       UnstructuredGridFilters, UniformGridFilters)
 from .grid import Grid, RectilinearGrid, UniformGrid
 from .objects import Table, Texture
-from .pointset import PointGrid, PolyData, StructuredGrid, UnstructuredGrid
+from .pointset import PointGrid, PolyData, StructuredGrid, UnstructuredGrid, ExplicitStructuredGrid
 from .pyvista_ndarray import pyvista_ndarray
