@@ -1,20 +1,18 @@
 """Supporting functions for polydata and grid objects."""
 
+import os
 import collections.abc
 import enum
 import logging
 import signal
 import sys
-import warnings
 from threading import Thread
 import threading
 import traceback
 
 import numpy as np
-import scooby
-import vtk
-import vtk.util.numpy_support as nps
 
+from pyvista import _vtk
 import pyvista
 from .fileio import from_meshio
 from . import transformations
@@ -23,10 +21,10 @@ from . import transformations
 class FieldAssociation(enum.Enum):
     """Represents which type of vtk field a scalar or vector array is associated with."""
 
-    POINT = vtk.vtkDataObject.FIELD_ASSOCIATION_POINTS
-    CELL = vtk.vtkDataObject.FIELD_ASSOCIATION_CELLS
-    NONE = vtk.vtkDataObject.FIELD_ASSOCIATION_NONE
-    ROW = vtk.vtkDataObject.FIELD_ASSOCIATION_ROWS
+    POINT = _vtk.vtkDataObject.FIELD_ASSOCIATION_POINTS
+    CELL = _vtk.vtkDataObject.FIELD_ASSOCIATION_CELLS
+    NONE = _vtk.vtkDataObject.FIELD_ASSOCIATION_NONE
+    ROW = _vtk.vtkDataObject.FIELD_ASSOCIATION_ROWS
 
 
 def get_vtk_type(typ):
@@ -39,7 +37,7 @@ def get_vtk_type(typ):
         int : the integer type id specified in vtkType.h
 
     """
-    typ = nps.get_vtk_array_type(typ)
+    typ = _vtk.get_vtk_array_type(typ)
     # This handles a silly string type bug
     if typ == 3:
         return 13
@@ -48,7 +46,7 @@ def get_vtk_type(typ):
 
 def vtk_bit_array_to_char(vtkarr_bint):
     """Cast vtk bit array to a char array."""
-    vtkarr = vtk.vtkCharArray()
+    vtkarr = _vtk.vtkCharArray()
     vtkarr.DeepCopy(vtkarr_bint)
     return vtkarr
 
@@ -67,7 +65,7 @@ def convert_string_array(arr, name=None):
 
     """
     if isinstance(arr, np.ndarray):
-        vtkarr = vtk.vtkStringArray()
+        vtkarr = _vtk.vtkStringArray()
         ########### OPTIMIZE ###########
         for val in arr:
             vtkarr.InsertNextValue(val)
@@ -115,27 +113,27 @@ def convert_array(arr, name=None, deep=0, array_type=None):
         else:
             # This will handle numerical data
             arr = np.ascontiguousarray(arr)
-            vtk_data = nps.numpy_to_vtk(num_array=arr, deep=deep, array_type=array_type)
-
+            vtk_data = _vtk.numpy_to_vtk(num_array=arr, deep=deep,
+                                         array_type=array_type)
         if isinstance(name, str):
             vtk_data.SetName(name)
         return vtk_data
     # Otherwise input must be a vtkDataArray
-    if not isinstance(arr, (vtk.vtkDataArray, vtk.vtkBitArray, vtk.vtkStringArray)):
+    if not isinstance(arr, (_vtk.vtkDataArray, _vtk.vtkBitArray, _vtk.vtkStringArray)):
         raise TypeError(f'Invalid input array type ({type(arr)}).')
     # Handle booleans
-    if isinstance(arr, vtk.vtkBitArray):
+    if isinstance(arr, _vtk.vtkBitArray):
         arr = vtk_bit_array_to_char(arr)
     # Handle string arrays
-    if isinstance(arr, vtk.vtkStringArray):
+    if isinstance(arr, _vtk.vtkStringArray):
         return convert_string_array(arr)
     # Convert from vtkDataArry to NumPy
-    return nps.vtk_to_numpy(arr)
+    return _vtk.vtk_to_numpy(arr)
 
 
 def is_pyvista_dataset(obj):
     """Return True if the Object is a PyVista wrapped dataset."""
-    return isinstance(obj, (pyvista.Common, pyvista.MultiBlock))
+    return isinstance(obj, (pyvista.DataSet, pyvista.MultiBlock))
 
 
 def point_array(mesh, name):
@@ -172,7 +170,7 @@ def parse_field_choice(field):
             field = FieldAssociation.POINT
         elif field in ['field', 'f', 'fields']:
             field = FieldAssociation.NONE
-        elif field in ['row', 'r',]:
+        elif field in ['row', 'r']:
             field = FieldAssociation.ROW
         else:
             raise ValueError(f'Data field ({field}) not supported.')
@@ -203,7 +201,7 @@ def get_array(mesh, name, preference='cell', info=False, err=False):
         Boolean to control whether to throw an error if array is not present.
 
     """
-    if isinstance(mesh, vtk.vtkTable):
+    if isinstance(mesh, _vtk.vtkTable):
         arr = row_array(mesh, name)
         if arr is None and err:
             raise KeyError(f'Data array ({name}) not present in this dataset.')
@@ -253,28 +251,48 @@ def get_array(mesh, name, preference='cell', info=False, err=False):
 
 
 def vtk_points(points, deep=True):
-    """Convert numpy points to a vtkPoints object."""
-    if not points.flags['C_CONTIGUOUS']:
-        points = np.ascontiguousarray(points)
-    vtkpts = vtk.vtkPoints()
-    vtkpts.SetData(nps.numpy_to_vtk(points, deep=deep))
+    """Convert numpy array or array-like to a vtkPoints object."""
+    points = np.asanyarray(points)
+
+    # verify is numeric
+    if not np.issubdtype(points.dtype, np.number):
+        raise TypeError('Points must be a numeric type')
+
+    # check dimensionality
+    if points.ndim == 1:
+        points = points.reshape(-1, 3)
+    elif points.ndim > 2:
+        raise ValueError('Dimension of ``points`` should be 1 or 2, not '
+                         f'{points.ndim}')
+
+    # verify shape
+    if points.shape[1] != 3:
+        raise ValueError('Points array must contain three values per point. '
+                         f'Shape is {points.shape} and should be (X, 3)')
+
+    # points must be contiguous
+    points = np.require(points, requirements=['C'])
+    vtkpts = _vtk.vtkPoints()
+    vtk_arr = _vtk.numpy_to_vtk(points, deep=deep)
+    vtkpts.SetData(vtk_arr)
     return vtkpts
 
 
 def line_segments_from_points(points):
     """Generate non-connected line segments from points.
 
-    Assumes points are ordered as line segments and an even number of points
-    are
+    Assumes points are ordered as line segments and an even number of
+    points.
 
     Parameters
     ----------
     points : np.ndarray
-        Points representing line segments. An even number must be given as
-        every two vertices represent a single line segment. For example, two
-        line segments would be represented as:
+        Points representing line segments. An even number must be
+        given as every two vertices represent a single line
+        segment. For example, two line segments would be represented
+        as:
 
-        np.array([[0, 0, 0], [1, 0, 0], [1, 0, 0], [1, 1, 0]])
+        ``np.array([[0, 0, 0], [1, 0, 0], [1, 0, 0], [1, 1, 0]])``
 
     Returns
     -------
@@ -283,13 +301,13 @@ def line_segments_from_points(points):
 
     Examples
     --------
-    This example plots two line segments at right angles to each other line.
+    This example plots two line segments at right angles to each other.
 
     >>> import pyvista
     >>> import numpy as np
     >>> points = np.array([[0, 0, 0], [1, 0, 0], [1, 0, 0], [1, 1, 0]])
     >>> lines = pyvista.lines_from_points(points)
-    >>> lines.plot() # doctest:+SKIP
+    >>> cpos = lines.plot()
 
     """
     if len(points) % 2 != 0:
@@ -332,7 +350,7 @@ def lines_from_points(points, close=False):
     cells[:, 1] = np.arange(0, len(points)-1, dtype=np.int_)
     cells[:, 2] = np.arange(1, len(points), dtype=np.int_)
     if close:
-        cells = np.append(cells, [[2, len(points)-1, 0],], axis=0)
+        cells = np.append(cells, [[2, len(points)-1, 0]], axis=0)
     poly.lines = cells
     return poly
 
@@ -352,7 +370,7 @@ def make_tri_mesh(points, faces):
         triangle mesh.
 
     faces : np.ndarray
-        Array of indices with shape (M, 3) containing the triangle
+        Array of indices with shape ``(M, 3)`` containing the triangle
         indices.
 
     Returns
@@ -373,7 +391,7 @@ def make_tri_mesh(points, faces):
     >>> faces = np.array([[0, 1, 4], [4, 7, 6], [2, 5, 4], [4, 5, 8],
     ...                   [0, 4, 3], [3, 4, 6], [1, 2, 4], [4, 8, 7]])
     >>> tri_mesh = pyvista.make_tri_mesh(points, faces)
-    >>> tri_mesh.plot(show_edges=True) # doctest:+SKIP
+    >>> cpos = tri_mesh.plot(show_edges=True)
 
     """
     if points.shape[1] != 3:
@@ -406,8 +424,8 @@ def vector_poly_data(orig, vec):
         raise ValueError('vec array must be 3D')
 
     # Create vtk points and cells objects
-    vpts = vtk.vtkPoints()
-    vpts.SetData(nps.numpy_to_vtk(np.ascontiguousarray(orig), deep=True))
+    vpts = _vtk.vtkPoints()
+    vpts.SetData(_vtk.numpy_to_vtk(np.ascontiguousarray(orig), deep=True))
 
     npts = orig.shape[0]
     cells = np.empty((npts, 2), dtype=pyvista.ID_TYPE)
@@ -416,13 +434,13 @@ def vector_poly_data(orig, vec):
     vcells = pyvista.utilities.cells.CellArray(cells, npts)
 
     # Create vtkPolyData object
-    pdata = vtk.vtkPolyData()
+    pdata = _vtk.vtkPolyData()
     pdata.SetPoints(vpts)
     pdata.SetVerts(vcells)
 
     # Add vectors to polydata
     name = 'vectors'
-    vtkfloat = nps.numpy_to_vtk(np.ascontiguousarray(vec), deep=True)
+    vtkfloat = _vtk.numpy_to_vtk(np.ascontiguousarray(vec), deep=True)
     vtkfloat.SetName(name)
     pdata.GetPointData().AddArray(vtkfloat)
     pdata.GetPointData().SetActiveVectors(name)
@@ -430,7 +448,7 @@ def vector_poly_data(orig, vec):
     # Add magnitude of vectors to polydata
     name = 'mag'
     scalars = (vec * vec).sum(1)**0.5
-    vtkfloat = nps.numpy_to_vtk(np.ascontiguousarray(scalars), deep=True)
+    vtkfloat = _vtk.numpy_to_vtk(np.ascontiguousarray(scalars), deep=True)
     vtkfloat.SetName(name)
     pdata.GetPointData().AddArray(vtkfloat)
     pdata.GetPointData().SetActiveScalars(name)
@@ -459,9 +477,9 @@ def array_from_vtkmatrix(matrix):
         Returned ndarray has shape (3, 3) or (4, 4) as appropriate.
 
     """
-    if isinstance(matrix, vtk.vtkMatrix3x3):
+    if isinstance(matrix, _vtk.vtkMatrix3x3):
         shape = (3, 3)
-    elif isinstance(matrix, vtk.vtkMatrix4x4):
+    elif isinstance(matrix, _vtk.vtkMatrix4x4):
         shape = (4, 4)
     else:
         raise TypeError('Expected vtk.vtkMatrix3x3 or vtk.vtkMatrix4x4 input,'
@@ -486,9 +504,9 @@ def vtkmatrix_from_array(array):
     """
     array = np.asarray(array)
     if array.shape == (3, 3):
-        matrix = vtk.vtkMatrix3x3()
+        matrix = _vtk.vtkMatrix3x3()
     elif array.shape == (4, 4):
-        matrix = vtk.vtkMatrix4x4()
+        matrix = _vtk.vtkMatrix4x4()
     else:
         raise ValueError(f'Invalid shape {array.shape}, must be (3, 3) or (4, 4).')
     m, n = array.shape
@@ -508,12 +526,13 @@ def is_meshio_mesh(mesh):
 
 
 def wrap(dataset):
-    """Wrap any given VTK data object to its appropriate PyVista data object.
+    """Wrap any given VTK data object to its appropriate pyvista data object.
 
     Other formats that are supported include:
     * 2D :class:`numpy.ndarray` of XYZ vertices
     * 3D :class:`numpy.ndarray` representing a volume. Values will be scalars.
     * 3D :class:`trimesh.Trimesh` mesh.
+    * 3D :class:`meshio` mesh.
 
     Parameters
     ----------
@@ -527,7 +546,7 @@ def wrap(dataset):
 
     Examples
     --------
-    Wrap a numpy array representing a random point cloud
+    Wrap a numpy array representing a random point cloud.
 
     >>> import numpy as np
     >>> import pyvista
@@ -542,7 +561,7 @@ def wrap(dataset):
       Z Bounds: 2.346e-03, 9.640e-01
       N Arrays: 0
 
-    Wrap a Trimesh object
+    Wrap a Trimesh object.
 
     >>> import trimesh
     >>> import pyvista
@@ -559,7 +578,7 @@ def wrap(dataset):
       Z Bounds: 0.000e+00, 1.000e+00
       N Arrays: 0
 
-    Wrap a VTK object
+    Wrap a VTK object.
 
     >>> import pyvista
     >>> import vtk
@@ -583,23 +602,14 @@ def wrap(dataset):
       N Arrays: 0
 
     """
-    wrappers = {
-        'vtkUnstructuredGrid': pyvista.UnstructuredGrid,
-        'vtkRectilinearGrid': pyvista.RectilinearGrid,
-        'vtkStructuredGrid': pyvista.StructuredGrid,
-        'vtkPolyData': pyvista.PolyData,
-        'vtkImageData': pyvista.UniformGrid,
-        'vtkStructuredPoints': pyvista.UniformGrid,
-        'vtkMultiBlockDataSet': pyvista.MultiBlock,
-        'vtkTable': pyvista.Table,
-        # 'vtkParametricSpline': pyvista.Spline,
-    }
-    # Otherwise, we assume a VTK data object was passed
-    if hasattr(dataset, 'GetClassName'):
-        key = dataset.GetClassName()
-    elif dataset is None:
-        return None
-    elif isinstance(dataset, np.ndarray):
+    # Return if None
+    if dataset is None:
+        return
+
+    # Check if dataset is a numpy array.  We do this first since
+    # pyvista_ndarray contains a VTK type that we don't want to
+    # directly wrap.
+    if isinstance(dataset, (np.ndarray, pyvista.pyvista_ndarray)):
         if dataset.ndim == 1 and dataset.shape[0] == 3:
             return pyvista.PolyData(dataset)
         if dataset.ndim > 1 and dataset.ndim < 3 and dataset.shape[1] == 3:
@@ -610,25 +620,45 @@ def wrap(dataset):
             mesh.active_scalars_name = 'values'
             return mesh
         else:
-            print(dataset.shape, dataset)
-            raise NotImplementedError('NumPy array could not be converted to PyVista.')
-    elif is_meshio_mesh(dataset):
+            raise NotImplementedError('NumPy array could not be wrapped pyvista.')
+
+    wrappers = {
+        'vtkExplicitStructuredGrid': pyvista.ExplicitStructuredGrid,
+        'vtkUnstructuredGrid': pyvista.UnstructuredGrid,
+        'vtkRectilinearGrid': pyvista.RectilinearGrid,
+        'vtkStructuredGrid': pyvista.StructuredGrid,
+        'vtkPolyData': pyvista.PolyData,
+        'vtkImageData': pyvista.UniformGrid,
+        'vtkStructuredPoints': pyvista.UniformGrid,
+        'vtkMultiBlockDataSet': pyvista.MultiBlock,
+        'vtkTable': pyvista.Table,
+        # 'vtkParametricSpline': pyvista.Spline,
+    }
+
+    # Check if a dataset is a VTK type
+    if hasattr(dataset, 'GetClassName'):
+        key = dataset.GetClassName()
+        try:
+            return wrappers[key](dataset)
+        except KeyError:
+            logging.warning(f'VTK data type ({key}) is not currently supported by pyvista.')
+        return
+
+    # wrap meshio
+    if is_meshio_mesh(dataset):
         return from_meshio(dataset)
-    elif dataset.__class__.__name__ == 'Trimesh':
+
+    # wrap trimesh
+    if dataset.__class__.__name__ == 'Trimesh':
         # trimesh doesn't pad faces
         n_face = dataset.faces.shape[0]
         faces = np.empty((n_face, 4), dataset.faces.dtype)
         faces[:, 1:] = dataset.faces
         faces[:, 0] = 3
         return pyvista.PolyData(np.asarray(dataset.vertices), faces)
-    else:
-        raise NotImplementedError(f'Type ({type(dataset)}) not able to be wrapped into a PyVista mesh.')
-    try:
-        wrapped = wrappers[key](dataset)
-    except KeyError:
-        logging.warning(f'VTK data type ({key}) is not currently supported by pyvista.')
-        return dataset  # if not supported just passes the VTK data object
-    return wrapped
+
+    # otherwise, flag tell the user we can't wrap this object
+    raise NotImplementedError(f'Unable to wrap ({type(dataset)}) into a pyvista type.')
 
 
 def image_to_texture(image):
@@ -690,16 +720,16 @@ def fit_plane_to_points(points, return_meta=False):
 
 def raise_not_matching(scalars, mesh):
     """Raise exception about inconsistencies."""
-    if isinstance(mesh, vtk.vtkTable):
+    if isinstance(mesh, _vtk.vtkTable):
         raise ValueError(f'Number of scalars ({scalars.size}) must match number of rows ({mesh.n_rows}).')
-    raise ValueError(f'Number of scalars ({scalars.size}) ' +
-                     f'must match either the number of points ({mesh.n_points}) ' +
+    raise ValueError(f'Number of scalars ({scalars.size}) '
+                     f'must match either the number of points ({mesh.n_points}) '
                      f'or the number of cells ({mesh.n_cells}).')
 
 
 def generate_plane(normal, origin):
-    """Return a vtk.vtkPlane."""
-    plane = vtk.vtkPlane()
+    """Return a _vtk.vtkPlane."""
+    plane = _vtk.vtkPlane()
     # NORMAL MUST HAVE MAGNITUDE OF 1
     normal = normal / np.linalg.norm(normal)
     plane.SetNormal(normal)
@@ -731,15 +761,15 @@ def check_depth_peeling(number_of_peels=100, occlusion_ratio=0.0):
 
     """
     # Try Depth Peeling with a basic scene
-    source = vtk.vtkSphereSource()
-    mapper = vtk.vtkPolyDataMapper()
+    source = _vtk.vtkSphereSource()
+    mapper = _vtk.vtkPolyDataMapper()
     mapper.SetInputConnection(source.GetOutputPort())
-    actor = vtk.vtkActor()
+    actor = _vtk.vtkActor()
     actor.SetMapper(mapper)
     # requires opacity < 1
     actor.GetProperty().SetOpacity(0.5)
-    renderer = vtk.vtkRenderer()
-    renderWindow = vtk.vtkRenderWindow()
+    renderer = _vtk.vtkRenderer()
+    renderWindow = _vtk.vtkRenderWindow()
     renderWindow.AddRenderer(renderer)
     renderWindow.SetOffScreenRendering(True)
     renderWindow.SetAlphaBitPlanes(True)
@@ -794,7 +824,7 @@ class ProgressMonitor():
             from tqdm import tqdm
         except ImportError:
             raise ImportError("Please install `tqdm` to monitor algorithms.")
-        self.event_type = vtk.vtkCommand.ProgressEvent
+        self.event_type = _vtk.vtkCommand.ProgressEvent
         self.progress = 0.0
         self._last_progress = self.progress
         self.algorithm = algorithm
@@ -913,3 +943,72 @@ def axis_rotation(points, angle, inplace=False, deg=True, axis='z'):
 
     rot_mat = transformations.axis_angle_rotation(axis_to_vec[axis], angle, deg=deg)
     return transformations.apply_transformation_to_points(rot_mat, points, inplace=inplace)
+
+
+def cubemap(path='', prefix='', ext='.jpg'):
+    """Construct a cubemap from 6 images.
+
+    Each of the 6 images must be in the following format:
+
+    - <prefix>negx<ext>
+    - <prefix>negy<ext>
+    - <prefix>negz<ext>
+    - <prefix>posx<ext>
+    - <prefix>posy<ext>
+    - <prefix>posz<ext>
+
+    Prefix may be empty, and extension will default to ``'.jpg'``
+
+    For example, if you have 6 images with the skybox2 prefix:
+
+    - ``'skybox2-negx.jpg'``
+    - ``'skybox2-negy.jpg'``
+    - ``'skybox2-negz.jpg'``
+    - ``'skybox2-posx.jpg'``
+    - ``'skybox2-posy.jpg'``
+    - ``'skybox2-posz.jpg'``
+
+    Parameters
+    ----------
+    prefix : str, optional
+        Prefix to the filename.
+
+    ext : str, optional
+        The filename extension.  For example ``'.jpg'``.
+
+    path : str, optional
+        Directory containing the cubemap images.
+
+    Returns
+    -------
+    pyvista.Texture
+        Texture with cubemap.
+
+    Examples
+    --------
+    >>> import pyvista
+    >>> skybox = pyvista.cubemap('my_directory', 'skybox', '.jpeg')  # doctest:+SKIP
+    """
+    sets = ['posx', 'negx', 'posy', 'negy', 'posz', 'negz']
+    image_paths = [os.path.join(path, f'{prefix}{suffix}{ext}') for suffix in sets]
+
+    for image_path in image_paths:
+        if not os.path.isfile(image_path):
+            file_str = '\n'.join(image_paths)
+            raise FileNotFoundError(f'Unable to locate {image_path}\n'
+                                    'Expected to find the following files:\n'
+                                    f'{file_str}')
+
+    texture = pyvista.Texture()
+    texture.cube_map = True  # Must be set prior to setting images
+
+    # add each image to the cubemap
+    for i, fn in enumerate(image_paths):
+        image = pyvista.read(fn)
+        flip = _vtk.vtkImageFlip()
+        flip.SetInputDataObject(image)
+        flip.SetFilteredAxis(1)  # flip y axis
+        flip.Update()
+        texture.SetInputDataObject(i, flip.GetOutput())
+
+    return texture
